/*
 Copyright (C) AC SOFTWARE SP. Z O.O.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef EXTRAS_PORTING_LINUX_SUPLA_SENSOR_SENSOR_PARSED_H_
#define EXTRAS_PORTING_LINUX_SUPLA_SENSOR_SENSOR_PARSED_H_

#include <supla/parser/parser.h>

#include <supla-common/proto.h>
#include <map>
#include <string>

namespace Supla {
namespace Sensor {

const char BatteryLevel[] = "battery_level";
const char MultiplierBatteryLevel[] = "multiplier_battery_level";


template <typename T> class SensorParsed : public T {
 public:
  explicit SensorParsed(Supla::Parser::Parser *);

  void setMapping(const std::string &parameter, const std::string &key);

  void setMapping(const std::string &parameter, const int index);

  void setMultiplier(const std::string &parameter, double multiplier);

  bool refreshParserSource();

  bool isParameterConfigured(const std::string &parameter);

  void handleGetChannelState(TDSC_ChannelState *channelState) override;

 protected:
  double getParameterValue(const std::string &parameter);

  Supla::Parser::Parser *parser = nullptr;
  std::map<std::string, std::string> parameterToKey;
  std::map<std::string, double> parameterMultiplier;
  int id;
};
template <typename T>
SensorParsed<T>::SensorParsed(Supla::Parser::Parser *parser)
    : parser(parser) {
  static int instanceCounter = 0;
  id = instanceCounter++;
}

template <typename T>
void SensorParsed<T>::setMapping(const std::string &parameter,
                                                const std::string &key) {
  parameterToKey[parameter] = key;
  parser->addKey(key, -1);  // ignore index
}

template <typename T>
void SensorParsed<T>::setMapping(const std::string &parameter,
                                                const int index) {
  std::string key = parameter;
  key += "_";
  key += std::to_string(id);
  parameterToKey[parameter] = key;
  parser->addKey(key, index);
}

template <typename T>
void SensorParsed<T>::setMultiplier(const std::string &parameter,
                                                   double multiplier) {
  parameterMultiplier[parameter] = multiplier;
}

template <typename T>
double SensorParsed<T>::getParameterValue(
    const std::string &parameter) {
  double multiplier = 1;
  if (parameterMultiplier.count(parameter)) {
    multiplier = parameterMultiplier[parameter];
  }
  return parser->getValue(parameterToKey[parameter]) * multiplier;
}

template <typename T>
bool SensorParsed<T>::refreshParserSource() {
  if (parser && parser->refreshParserSource()) {
    return true;
  }
  return false;
}

template <typename T>
bool SensorParsed<T>::isParameterConfigured(
    const std::string &parameter) {
  return parameterToKey.count(parameter) > 0;
}

template <typename T>
void SensorParsed<T>::handleGetChannelState(TDSC_ChannelState *channelState) {
  unsigned char batteryLevel = 255;
  if (isParameterConfigured(BatteryLevel)) {
    if (refreshParserSource()) {
      batteryLevel = getParameterValue(BatteryLevel);
    }
    if (T::getChannel()) {
      T::getChannel()->setBatteryLevel(batteryLevel);
    }
    if (T::getSecondaryChannel()) {
      T::getSecondaryChannel()->setBatteryLevel(batteryLevel);
    }
  }

  T::handleGetChannelState(channelState);
}

};  // namespace Sensor
};  // namespace Supla

#endif  // EXTRAS_PORTING_LINUX_SUPLA_SENSOR_SENSOR_PARSED_H_
